from fast_arrow.api_requestor import get
from fast_arrow import util
from fast_arrow.resources.option import Option


class OptionOrder(object):

    @classmethod
    def all(cls, token):
        """
        fetch all option positions
        """
        url = 'https://api.robinhood.com/options/orders/'
        data = get(url, token=token)
        results = data["results"]
        while data["next"]:
            data = get(data["next"], token)
            results.extend(data["results"])
        return results

    @classmethod
    def humanize_numbers(cls, option_orders):
        results = []
        for oo in option_orders:
            keys_to_humanize = ["processed_premium"]
            coef = (1.0 if oo["direction"] == "credit" else -1.0)
            for k in keys_to_humanize:
                if oo[k] == None:
                    continue
                oo[k] = float(oo[k]) * coef
            results.append(oo)
        return results



from fast_arrow.resources.auth import Auth

from fast_arrow.resources.option_chain import OptionChain
from fast_arrow.resources.option_event import OptionEvent
from fast_arrow.resources.option_order import OptionOrder
from fast_arrow.resources.option_marketdata import OptionMarketdata
from fast_arrow.resources.option_position import OptionPosition
from fast_arrow.resources.option import Option

from fast_arrow.resources.stock_order import StockOrder
from fast_arrow.resources.stock_marketdata import StockMarketdata
from fast_arrow.resources.stock_position import StockPosition
from fast_arrow.resources.stock import Stock

from fast_arrow.resources.user import User



from fast_arrow.api_requestor import get
from fast_arrow.util import chunked_list


class OptionMarketdata(object):

    @classmethod
    def quote_by_instrument_id(cls, bearer, _id):
        return cls.quotes_by_instrument_ids(bearer, [_id])[0]


    @classmethod
    def quote_by_instrument_url(cls, bearer, url):
        return cls.quotes_by_instrument_urls(bearer, [url])[0]


    @classmethod
    def quotes_by_instrument_ids(cls, bearer, ids):
        """
        create instrument urls, fetch, return results
        """
        base_url = "https://api.robinhood.com/options/instruments/"
        id_urls = ["{}{}/".format(base_url, _id) for _id in ids]
        return cls.quotes_by_instrument_urls(bearer, id_urls)


    @classmethod
    def quotes_by_instrument_urls(cls, bearer, urls):
        """
        fetch and return results
        note - data requests are batched to limit urls to 50 per http request
        """
        results = []

        for _urls in chunked_list(urls, 50):
            url = "https://api.robinhood.com/marketdata/options/"
            params = {"instruments": ",".join(_urls)}
            data = get(url, bearer=bearer, params=params)

            partial_results = data["results"]

            while ("next" in data and data["next"]):
                data = get(data["next"], bearer=bearer)
                partial_results.extend(data["results"])

            results.extend(partial_results)

        return results



from fast_arrow.api_requestor import get


class Stock(object):

    @classmethod
    def fetch(cls, bearer, symbol):
        """
        fetch data for stock
        """
        assert(type(symbol) is str)

        url = "https://api.robinhood.com/instruments/?symbol={0}".format(symbol)
        data = get(url, bearer=bearer)
        return data["results"][0]

    @classmethod
    def quote_by_instrument_id(cls, _id):
        return []

    @classmethod
    def all(cls, bearer, symbols):
        """"
        fetch data for multiple stocks
        """
        params = {"symbol": ",".join(symbols)}
        request_url = "https://api.robinhood.com/instruments/"

        data = get(request_url, bearer=bearer, params=params)
        results = data["results"]

        while data["next"]:
            data = get(data["next"], bearer=bearer)
            results.extend(data["results"])
        return results



from fast_arrow.api_requestor import get
from fast_arrow.util import chunked_list
from fast_arrow.resources.option_marketdata import OptionMarketdata


class Option(object):

    @classmethod
    def fetch_by_ids(cls, bearer, ids):
        results = []
        params = {"ids": ",".join(ids)}
        request_url = "https://api.robinhood.com/options/instruments/"
        data = get(request_url, bearer=bearer, params=params)
        results = data["results"]
        while data["next"]:
            data = get(data["next"], bearer=bearer)
            results.extend(data["results"])
        return results


    # deprecate me
    @classmethod
    def fetch(cls, bearer, _id):
        """
        fetch by instrument id
        """
        return cls.fetch_list(bearer, [_id])[0]


    # deprecate me
    @classmethod
    def fetch_list(cls, bearer, ids):
        """
        fetch instruments by ids
        """
        results = []
        request_url = "https://api.robinhood.com/options/instruments/"

        for _ids in chunked_list(ids, 50):

            params = {"ids": ",".join(_ids)}
            data = get(request_url, bearer=bearer, params=params)
            partial_results = data["results"]

            while data["next"]:
                data = get(data["next"], bearer=bearer)
                partial_results.extend(data["results"])
            results.extend(partial_results)

        return results


    @classmethod
    def in_chain(cls, bearer, chain_id, expiration_dates=[]):
        """
        fetch all option instruments in an options chain
        - expiration_dates = optionally scope
        """

        request_url = "https://api.robinhood.com/options/instruments/"
        params = {
            "chain_id": chain_id,
            "expiration_dates": ",".join(expiration_dates)
        }

        data = get(request_url, bearer=bearer, params=params)
        results = data['results']

        while data['next']:
            data = get(data['next'], bearer=bearer)
            results.extend(data['results'])
        return results


    @classmethod
    def mergein_marketdata_list(cls, bearer, options):
        ids = [x["id"] for x in options]
        mds = OptionMarketdata.quotes_by_instrument_ids(bearer, ids)

        results = []
        for o in options:
            # @TODO optimize this so it's better than O(n^2)
            md = [md for md in mds if md['instrument'] == o['url']][0]
            # there is overlap in keys, so it's fine to do a merge
            merged_dict = dict( list(o.items()) + list(md.items()) )
            results.append(merged_dict)

        return results



from fast_arrow.api_requestor import get
from fast_arrow import util
from fast_arrow.resources.option import Option
from fast_arrow.resources.option_marketdata import OptionMarketdata
from fast_arrow.util import chunked_list


class OptionPosition(object):

    @classmethod
    def all(cls, bearer):
        """
        fetch all option positions
        """
        url = 'https://api.robinhood.com/options/positions/'
        params = { }
        data = get(url, bearer=bearer, params=params)
        results = data["results"]
        while data["next"]:
            data = get(data["next"], token)
            results.extend(data["results"])
        return results


    @classmethod
    def append_marketdata(cls, bearer, option_position):
        """
        Fetch and merge in Marketdata for option position
        """
        return cls.append_marketdata_list(bearer, [option_position])[0]


    @classmethod
    def mergein_marketdata_list(cls, bearer, option_positions):
        """
        Fetch and merge in Marketdata for each option position
        """
        ids = cls._extract_ids(option_positions)
        mds = OptionMarketdata.quotes_by_instrument_ids(bearer, ids)

        results = []
        for op in option_positions:
            # @TODO optimize this so it's better than O(n^2)
            md = [x for x in mds if x['instrument'] == op['option']][0]
            # there is no overlap in keys so this is fine
            merged_dict = dict( list(op.items()) + list(md.items()) )
            results.append(merged_dict)
        return results


    @classmethod
    def mergein_instrumentdata_list(cls, bearer, option_positions):
        ids = cls._extract_ids(option_positions)
        idatas = Option.fetch_list(bearer, ids)

        results = []
        for op in option_positions:
            idata = [x for x in idatas if x['url'] == op['instrument']][0]
            # there is an overlap in keys, {'chain_symbol', 'url', 'type', 'created_at', 'id', 'updated_at', 'chain_id'}
            # @TODO this is ugly. let's fix it later
            # alternative method,
            #   wanted_keys = ['strike_price']
            #   idata_subset = dict((k, idata[k]) for k in wanted_keys if k in idata)
            merge_me = {
                "option_type": idata["type"],
                "strike_price": idata["strike_price"],
                "expiration_date": idata["expiration_date"],
                "min_ticks": idata["min_ticks"]
            }
            merged_dict = dict( list(op.items()) + list(merge_me.items()) )
            results.append(merged_dict)

        return results


    @classmethod
    def humanize_numbers(cls, option_positions):
        results = []
        for op in option_positions:
            keys_to_humanize = [
                "quantity",
                "delta",
                "theta",
                "gamma",
                "vega",
                "rho"]

            coef = (1.0 if op["type"] == "long" else -1.0)

            for k in keys_to_humanize:
                if op[k] == None:
                    continue
                op[k] = float(op[k]) * coef

            op["chance_of_profit"] = (op["chance_of_profit_long"] if op["type"] == "long" else op["chance_of_profit_short"])

            results.append(op)

        return results


    @classmethod
    def _extract_ids(cls, option_positions):
        ids = []
        for op in option_positions:
            _id = util.get_last_path(op["option"])
            ids.append(_id)
        return ids



import configparser
from urllib.parse import urlparse


def get_username_password(config_file):
    config = configparser.ConfigParser()
    config.read(config_file)
    return [config['account']['username'], config['account']['password']]


def get_last_path(url_string):
    o = urlparse(url_string)
    paths = o.path.rsplit("/")
    return list(filter(None, paths))[-1]


def chunked_list(_list, _chunk_size):
    """Yield successive n-sized chunks from l."""
    for i in range(0, len(_list), _chunk_size):
        yield _list[i:i + _chunk_size]



import fast_arrow
from fast_arrow.api_requestor import post


class Auth(object):
    @classmethod
    def login(cls, username, password, mfa_code=None):
        """
        Login with username, password, and optionally MFA
        """
        url = 'https://api.robinhood.com/api-token-auth/'

        payload = {
            'username': username,
            'password': password
        }
        if mfa_code:
            payload['mfa_code'] = mfa_code

        resj = post(url, payload=payload)

        if 'token' in resj.keys():
            return resj['token']


    @classmethod
    def get_oauth_token(cls, token):
        """
        get OAuth2 Bearer token from token
        """
        url = "https://api.robinhood.com/oauth2/migrate_token/"
        res = post(url, token=token)
        oauth_token = res["access_token"]
        return oauth_token


    @classmethod
    def logout(cls, token):
        """
        Logout for given token
        """
        url = 'https://api.robinhood.com/api-token-logout/'
        data = post(url, token=token, timeout=15)
        return data



import requests


def gen_headers(token, bearer):
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "en;q=1, fr;q=0.9, de;q=0.8, ja;q=0.7, nl;q=0.6, it;q=0.5",
        "Content-Type": "application/x-www-form-urlencoded; charset=utf-8",
        "X-Robinhood-API-Version": "1.0.0",
        "Connection": "keep-alive",
        "User-Agent": "Robinhood/823 (iPhone; iOS 7.1.2; Scale/2.00)"
    }
    if token:
        headers["Authorization"] = "Token {0}".format(token)
    if bearer:
        headers["Authorization"] = "Bearer {0}".format(bearer)
    return headers


def get(url, token=None, bearer=None, params={}):
    """
    Execute HTTP GET
    """
    headers = gen_headers(token, bearer)
    res = requests.get(url, headers=headers, params=params, timeout=15)
    return res.json()


def post(url, token=None, bearer=None, payload=None):
    """
    Execute HTTP POST
    """
    headers = gen_headers(token, bearer)
    res = requests.post(url, headers=headers, data=payload, timeout=15)
    return res.json()



from fast_arrow.api_requestor import get
from fast_arrow.util import chunked_list


class StockMarketdata(object):

    @classmethod
    def quote_by_symbol(cls, symbol):
        """
        fetch and return results
        """
        return []

    @classmethod
    def quote_by_symbols(cls, symbols):
        """
        fetch and return results
        """
        return []


    @classmethod
    def quotes_by_instrument_ids(cls, bearer, ids):
        """
        create instrument urls, fetch, return results
        """
        base_url = "https://api.robinhood.com/options/instruments/"
        id_urls = ["{}{}/".format(base_url, _id) for _id in ids]
        return cls.quotes_by_instrument_urls(bearer, id_urls)


    @classmethod
    def quotes_by_instrument_urls(cls, bearer, urls):
        """
        fetch and return results
        """
        instruments = ",".join(urls)
        params = {"instruments": instruments}
        data = get(url, bearer=bearer)
        results = data["results"]
        while data["next"]:
            data = get(data["next"], bearer=bearer)
            results.extend(data["results"])
        return results



from fast_arrow.api_requestor import get


class OptionChain(object):

    @classmethod
    def fetch(cls, bearer, id):
        """
        fetch option chain for instrument
        """
        url = "https://api.robinhood.com/options/chains/"
        params = {
            "equity_instrument_ids": [id],
            "state": "active",
            "tradability": "tradable"
        }
        data = get(url, bearer=bearer, params=params)
        return data['results'][0]



import os
import sys
from setuptools import setup
from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass into pytest")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = '-n auto'

    def run_tests(self):
        import shlex
        import pytest
        errno = pytest.main(shlex.split(self.pytest_args))
        sys.exit(errno)


with open("README.md", "r") as fh:
    long_description = fh.read()

deps = [
    'click',
    'pathlib2',
    'requests',
    'yarl']


test_deps = [
    'pipenv',
    'pytest',
    'pytest-cov',
    'detox',
    'flake8',
    'vcrpy']

setup(name='fast_arrow',
    version='0.1.0',
    description='API client for Robinhood',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Weston Platter',
    author_email='westonplatter@gmail.com',
    url='https://github.com/westonplatter/fast_arrow/',
    license='MIT License',
    python_requires=">=3.5",
    packages=['fast_arrow', 'fast_arrow.resources'],
    install_requires=deps,
    tests_require=test_deps,
    cmdclass={'test': PyTest},
    project_urls={
        'Issue Tracker': 'https://github.com/westonplatter/fast_arrow/issues',
        'Source Code': 'https://github.com/westonplatter/fast_arrow',
    }
)



from fast_arrow.api_requestor import get


class Position(object):

    @classmethod
    def all(cls, token):
        """
        fetch all positions
        """
        url = 'https://api.robinhood.com/positions/'
        positions = []

        data = get(url, token)
        positions.extend(data["results"])
        while data["next"]:
            data = get(data["next"], token)
            positions.extend(data["results"])
        return positions



# from __future__ import absolute_import, division, print_function

import datetime
import json

from fast_arrow import api_requestor, util

class FastArrowObject(dict):

    def __init__(self, id=None, api_key=None, **params):
        super(FastArrowObject, self).__init__()
        self.api_key = api_key

    def request(self, method, url, params=None):
        if params is None:
        requestor = api_requestor.ApiRequestor(
            key=self.api_key,
            api_base=self.api_base())
        response = requestor.request(method, url, params)

        return util.convert_to_fa_object(response)



from fast_arrow.api_requestor import get


class User(object):

    @classmethod
    def fetch(cls, token):
        url = 'https://api.robinhood.com/user/'
        resj = get(url, token=token)
        return resj



from fast_arrow.api_requestor import get


class StockPosition(object):

    @classmethod
    def all(cls, bearer):
        url = "https://api.robinhood.com/positions/"
        data = get(url, bearer=bearer)
        results = data["results"]
        while data["next"]:
            data = get(data["next"], token=token)
            results.extend(data["results"])
        return results
